Class: Torque::PostgreSQL::Attributes::SimpleEnum::Methods

Inherits:
Module
  • Object
show all
Defined in:
lib/torque/postgresql/attributes/simple_enum.rb

Overview

Replaces the module that Rails uses to generate the methods of an enum, so that only the ones that make sense here are defined

Instance Method Summary collapse

Constructor Details

#initialize(klass) ⇒ Methods

Returns a new instance of Methods.



15
16
17
# File 'lib/torque/postgresql/attributes/simple_enum.rb', line 15

def initialize(klass)
  @klass = klass
end